@@ -27,6 +27,7 @@ from eventlet import greenio, GreenPool, sleep, wsgi, listen
 from paste.deploy import loadapp, appconfig
 from eventlet.green import socket, ssl
 from webob import Request
+from urllib import unquote
 
 from swift.common.utils import get_logger, drop_privileges, \
     validate_configuration, capture_stdio, NullLogger
@@ -68,11 +69,13 @@ def get_socket(conf, default_port=8080):
             if addr[0] in (socket.AF_INET, socket.AF_INET6)][0]
     sock = None
     retry_until = time.time() + 30
+    warn_ssl = False
     while not sock and time.time() < retry_until:
         try:
             sock = listen(bind_addr, backlog=int(conf.get('backlog', 4096)),
                         family=address_family)
             if 'cert_file' in conf:
+                warn_ssl = True
                 sock = ssl.wrap_socket(sock, certfile=conf['cert_file'],
                     keyfile=conf['key_file'])
         except socket.error, err:
@@ -86,6 +89,12 @@ def get_socket(conf, default_port=8080):
     # in my experience, sockets can hang around forever without keepalive
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
     sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 600)
+    if warn_ssl:
+        ssl_warning_message = 'WARNING: SSL should only be enabled for ' \
+                              'testing purposes. Use external SSL ' \
+                              'termination for a production deployment.'
+        get_logger(conf).warning(ssl_warning_message)
+        print _(ssl_warning_message)
     return sock
 
 
@@ -256,7 +265,10 @@ def make_pre_authed_request(env, method=None, path=None, body=None,
     :param method: HTTP method of new request; default is from
                    the original env.
     :param path: HTTP path of new request; default is from the
-                 original env.
+                 original env. path should be compatible with what you
+                 would send to Request.blank. path should be quoted and it
+                 can include a query string. for example:
+                 '/a%20space?unicode_str%E8%AA%9E=y%20es'
     :param body: HTTP body of new request; empty by default.
     :param headers: Extra HTTP headers of new request; None by
                     default.
@@ -268,17 +280,21 @@ def make_pre_authed_request(env, method=None, path=None, body=None,
                   have no HTTP_USER_AGENT.
     :returns: Fresh webob.Request object.
     """
-    newenv = make_pre_authed_env(env, method, path, agent)
+    query_string = None
+    if path and '?' in path:
+        path, query_string = path.split('?', 1)
+    newenv = make_pre_authed_env(env, method, path=unquote(path), agent=agent,
+                                 query_string=query_string)
     if not headers:
         headers = {}
     if body:
-        return Request.blank(path, environ=newenv, body=body,
-                             headers=headers)
+        return Request.blank(path, environ=newenv, body=body, headers=headers)
     else:
         return Request.blank(path, environ=newenv, headers=headers)
 
 
-def make_pre_authed_env(env, method=None, path=None, agent='Swift'):
+def make_pre_authed_env(env, method=None, path=None, agent='Swift',
+                        query_string=None):
     """
     Returns a new fresh WSGI environment with escalated privileges to
     do backend checks, listings, etc. that the remote user wouldn't
@@ -287,7 +303,14 @@ def make_pre_authed_env(env, method=None, path=None, agent='Swift'):
     :param env: The WSGI environment to base the new environment on.
     :param method: The new REQUEST_METHOD or None to use the
                    original.
-    :param path: The new PATH_INFO or None to use the original.
+    :param path: The new path_info or none to use the original. path
+                 should NOT be quoted. When building a url, a Webob
+                 Request (in accordance with wsgi spec) will quote
+                 env['PATH_INFO'].  url += quote(environ['PATH_INFO'])
+    :param query_string: The new query_string or none to use the original.
+                         When building a url, a Webob Request will append
+                         the query string directly to the url.
+                         url += '?' + env['QUERY_STRING']
     :param agent: The HTTP user agent to use; default 'Swift'. You
                   can put %(orig)s in the agent to have it replaced
                   with the original env's HTTP_USER_AGENT, such as
@@ -297,20 +320,18 @@ def make_pre_authed_env(env, method=None, path=None, agent='Swift'):
     :returns: Fresh WSGI environment.
     """
     newenv = {}
-    for name in ('eventlet.posthooks', 'HTTP_USER_AGENT',
+    for name in ('eventlet.posthooks', 'HTTP_USER_AGENT', 'HTTP_HOST',
                  'PATH_INFO', 'QUERY_STRING', 'REMOTE_USER', 'REQUEST_METHOD',
-                 'SCRIPT_NAME', 'SERVER_NAME', 'SERVER_PORT',
-                 'SERVER_PROTOCOL', 'swift.cache', 'swift.source',
-                 'swift.trans_id'):
+                 'SERVER_NAME', 'SERVER_PORT', 'SERVER_PROTOCOL',
+                 'swift.cache', 'swift.source', 'swift.trans_id'):
         if name in env:
             newenv[name] = env[name]
     if method:
         newenv['REQUEST_METHOD'] = method
     if path:
-        if '?' in path:
-            path, query_string = path.split('?', 1)
-            newenv['QUERY_STRING'] = query_string
         newenv['PATH_INFO'] = path
+    if query_string:
+        newenv['QUERY_STRING'] = query_string
     if agent:
         newenv['HTTP_USER_AGENT'] = (
             agent % {'orig': env.get('HTTP_USER_AGENT', '')}).strip()
